﻿#region License
/* NBit Bit Manipulation Library
   Copyright 2012 Illusory Studios LLC <james@illusorystudios.com>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

using System.Linq;
using System.Text;
using NUnit.Framework;

namespace NBit.Tests
{
    [TestFixture]
    public class CrcTest
    {
        [Test]
        public void Parity()
        {
            var parity = Crc.Parity;
            Assert.AreEqual(0, parity.Compute(new[] { (byte)0 }));
            Assert.AreEqual(1, parity.Compute(new[] { (byte)1 }));
            Assert.AreEqual(1, parity.Compute(new[] { (byte)2 }));
            Assert.AreEqual(0, parity.Compute(new[] { (byte)3 }));
        }

        // See: http://www.usb.org/developers/whitepapers/crcdes.pdf
        [Test]
        public void Crc5Usb()
        {
            var usb = Crc.Crc5Usb;
            Assert.AreEqual(0x19, usb.Compute("123456789"));

            uint sofExample = usb.Begin();
            sofExample = usb.Next(sofExample, 0x10, 8); // LSB first
            sofExample = usb.Next(sofExample, 0x07, 3);
            Assert.AreEqual(0x05, usb.End(sofExample));

            uint setupExample = usb.Begin();
            setupExample = usb.Next(setupExample, 0x15, 8);
            setupExample = usb.Next(setupExample, 0x07, 3);
            Assert.AreEqual(0x1d, usb.End(setupExample));
        }

        [Test]
        public void Crc8OneWire()
        {
            var ow = Crc.Crc8OneWire;
            Assert.AreEqual(0x1b, ow.Compute(new[] { (byte)0xaa, (byte)0xbb }));
            Assert.AreEqual(0x9c, ow.Compute(new[] { (byte)0xee, (byte)0xff }));
        }

        [Test]
        public void Crc14Kangaroo()
        {
            var k = Crc.Crc14Kangaroo; byte[] bytes;

            bytes = Encoding.ASCII.GetBytes("123456789");
            Assert.AreEqual(0x1d66, k.Compute(bytes, 0, bytes.Length, 7));

            bytes = bytes.Concat(new[] { (byte)0x66, (byte)0x3a }).ToArray();
            Assert.AreEqual(0x307e, k.Compute(bytes, 0, bytes.Length, 7));

            bytes = Encoding.ASCII.GetBytes("123");
            Assert.AreEqual(0x1255, k.Compute(bytes, 0, bytes.Length, 7));

            bytes = bytes.Concat(new[] { (byte)0x55, (byte)0x24 }).ToArray();
            Assert.AreEqual(0x307e, k.Compute(bytes, 0, bytes.Length, 7));
        }

        [Test]
        public void Crc15Can()
        {
            var can = Crc.Crc15Can;
            Assert.AreEqual(0x059e, can.Compute("123456789"));
        }

        [Test]
        public void Crc16()
        {
            var std = Crc.Crc16;
            Assert.AreEqual(0xf353, std.Compute("Hello"));
            Assert.AreEqual(0xbb3d, std.Compute("123456789"));
        }

        [Test]
        public void Crc16Ccitt()
        {
            var ccitt = Crc.Crc16Ccitt;
            Assert.AreEqual(0xffff, ccitt.Compute(""));
            Assert.AreEqual(0xdada, ccitt.Compute("Hello"));
            Assert.AreEqual(0x3bb2, ccitt.Compute("Goodbye"));
            Assert.AreEqual(0x29b1, ccitt.Compute("123456789"));
        }

        [Test]
        public void Crc16Modbus()
        {
            var modbus = Crc.Crc16Modbus;
            Assert.AreEqual(0xf377, modbus.Compute("Hello"));
            Assert.AreEqual(0x4b37, modbus.Compute("123456789"));
        }

        [Test]
        public void Crc16XModem()
        {
            var xmodem = Crc.Crc16XModem;
            Assert.AreEqual(0xcbd6, xmodem.Compute("Hello"));
            Assert.AreEqual(0x31c3, xmodem.Compute("123456789"));
        }

        [Test]
        public void Crc32()
        {
            var std = Crc.Crc32;
            Assert.AreEqual(0xf7d18982, std.Compute("Hello"));
            Assert.AreEqual(0xcbf43926, std.Compute("123456789"));
            Assert.AreEqual(0x414fa339, std.Compute("The quick brown fox jumps over the lazy dog"));
        }

        [Test]
        public void Crc32Castagnoli()
        {
            var c = Crc.Crc32Castagnoli;
            Assert.AreEqual(0xe3069283, c.Compute("123456789"));
        }

        [Test]
        public void KoopmanPolynomial()
        {
            Assert.AreEqual(0xbb3d, Crc.Custom(16, 0xc002, CrcFormat.LsbFirst | CrcFormat.KoopmanPolynomial, false, false).Compute("123456789"));
        }

        [Test]
        public void ReversedPolynomial()
        {
            Assert.AreEqual(0xbb3d, Crc.Custom(16, 0xa001, CrcFormat.LsbFirst | CrcFormat.ReversedPolynomial, false, false).Compute("123456789"));
        }
    }
}
