﻿#region License
/* NBit Bit Manipulation Library
   Copyright 2012 Illusory Studios LLC <james@illusorystudios.com>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

using System;

namespace NBit
{
    /// <summary>
    /// Provides bit manipulation extension methods for unsigned integers.
    /// Many of these algorithms can be found at Bit Twiddling Hacks: http://graphics.stanford.edu/~seander/bithacks.html
    /// </summary>
    public static class BitMathExtensions
    {
        /// <summary>
        /// Counts the number of one bits in a value.
        /// </summary>
        /// <param name="value">The value to count the bits of.</param>
        /// <returns>The number of one bits.</returns>
        public static int CountBits(this uint value)
        {
            int count;
            for (count = 0; value != 0; count++) { value &= value - 1; }
            return count;
        }

        /// <summary>
        /// Counts the number of trailing zeros in a value.
        /// </summary>
        /// <param name="value">The value to count the trailing zeros of.</param>
        /// <returns>The number of trailing zeros.</returns>
        public static int CountTrailingZeros(this uint value)
        {
            int count;
            if (value == 0) { return 32; }

            value = (value ^ (value - 1)) >> 1;
            for (count = 0; value != 0; count++) { value >>= 1; }
            return count;
        }

        /// <summary>
        /// Computes the parity of a value.
        /// </summary>
        /// <param name="value">The value to compute the parity of.</param>
        /// <returns>True if the parity is odd, or false if it is even.</returns>
        public static bool IsOddParity(this uint value)
        {
            value ^= value >> 1;
            value ^= value >> 2;
            value = (value & 0x11111111u) * 0x11111111u;
            return 0 != (value & (1u << 28));
        }

        /// <summary>
        /// Checks if a value is a power of two.
        /// </summary>
        /// <param name="value">The value to check.</param>
        /// <returns>True if the value is a power of two. Zero is not considered a power of two.</returns>
        public static bool IsPowerOf2(this uint value)
        {
            return value != 0 && (value & (value - 1)) == 0;
        }

        /// <summary>
        /// Reverses the bits of a value.
        /// </summary>
        /// <param name="value">The value to reverse the bits of.</param>
        /// <returns>The reversed bits.</returns>
        public static uint ReverseBits(this uint value)
        {
            return value.ReverseBits(32);
        }

        /// <summary>
        /// Masks and reverses the specified number of bits of a value.
        /// </summary>
        /// <param name="value">The value to reverse the bits of.</param>
        /// <param name="bits">The number of bits to reverse.</param>
        /// <returns>The masked and reversed bits.</returns>
        /// <exception cref="ArgumentOutOfRangeException"><paramref name="bits"/> must be between 0 and 32.</exception>
        public static uint ReverseBits(this uint value, int bits)
        {
            if (bits < 0 || bits > 32) { throw new ArgumentOutOfRangeException("bits"); }
            if (bits < 32) { value &= (1u << bits) - 1; }

            uint result = 0;
            for (int i = 0; i < bits; i++)
            {
                result <<= 1; result |= value & 1;
                value >>= 1;
            }
            return result;
        }

        /// <summary>
        /// Rounds a value up to the nearest power of two.
        /// </summary>
        /// <param name="value">The value to round up.</param>
        /// <returns>The rounded-up value. Values greater than 2^31 overflow to 0.</returns>
        public static uint RoundUpToPowerOf2(this uint value)
        {
            if (value == 0) { return 1; }

            value--;
            value |= value >> 1;
            value |= value >> 2;
            value |= value >> 4;
            value |= value >> 8;
            value |= value >> 16;
            value++;

            return value;
        }
    }
}
