﻿#region License
/* NBit Bit Manipulation Library
   Copyright 2012 Illusory Studios LLC <james@illusorystudios.com>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

using System;
using System.Text;

namespace NBit
{
    partial class Crc
    {
        static void CheckArrayRange(byte[] buffer, int offset, int count)
        {
            if (buffer == null) { throw new ArgumentNullException("buffer"); }
            if (offset < 0 || offset > buffer.Length - count) { throw new ArgumentOutOfRangeException("offset"); }
            if (count < 0 || count > buffer.Length) { throw new ArgumentOutOfRangeException("count"); }
        }

        /// <summary>
        /// Sends bits from a byte array through the CRC.
        /// </summary>
        /// <param name="crc">The current CRC value.</param>
        /// <param name="buffer">The array of bytes.</param>
        /// <param name="offset">The index of the first byte in <paramref name="buffer"/> to read.</param>
        /// <param name="count">The number of bytes to read.</param>
        /// <returns>The modified CRC.</returns>
        public uint Next(uint crc, byte[] buffer, int offset, int count)
        {
            CheckArrayRange(buffer, offset, count);
            for (int i = 0; i < count; i++) { crc = Next(crc, buffer[offset + i]); }
            return crc;
        }

        /// <summary>
        /// Sends bits from a byte array through the CRC.
        /// </summary>
        /// <param name="crc">The current CRC value.</param>
        /// <param name="buffer">The array of bytes.</param>
        /// <param name="offset">The index of the first byte in <paramref name="buffer"/> to read.</param>
        /// <param name="count">The number of bytes to read.</param>
        /// <param name="bits">The number of bits per byte to send.</param>
        /// <returns>The modified CRC.</returns>
        public uint Next(uint crc, byte[] buffer, int offset, int count, int bits)
        {
            if (bits == 8) { return Next(crc, buffer, offset, count); }

            CheckArrayRange(buffer, offset, count);
            for (int i = 0; i < count; i++) { crc = NextBits(crc, buffer[offset + i], bits); }
            return crc;
        }

        /// <summary>
        /// Computes a CRC from a byte array.
        /// </summary>
        /// <param name="buffer">The bytes to compute the CRC from.</param>
        /// <returns>The final CRC.</returns>
        public uint Compute(byte[] buffer)
        {
            if (buffer == null) { throw new ArgumentNullException("buffer"); }
            return Compute(buffer, 0, buffer.Length);
        }

        /// <summary>
        /// Computes a CRC from a byte array.
        /// </summary>
        /// <param name="buffer">The bytes to compute the CRC from.</param>
        /// <param name="offset">The index of the first byte in <paramref name="buffer"/> to read.</param>
        /// <param name="count">The number of bytes to read.</param>
        /// <returns>The final CRC.</returns>
        public uint Compute(byte[] buffer, int offset, int count)
        {
            return Compute(buffer, offset, count, 8);
        }

        /// <summary>
        /// Computes a CRC from a byte array.
        /// </summary>
        /// <param name="buffer">The bytes to compute the CRC from.</param>
        /// <param name="offset">The index of the first byte in <paramref name="buffer"/> to read.</param>
        /// <param name="count">The number of bytes to read.</param>
        /// <param name="bits">The number of bits of the byte to use.</param>
        /// <returns>The final CRC.</returns>
        public uint Compute(byte[] buffer, int offset, int count, int bits)
        {
            uint crc = Begin();
            crc = Next(crc, buffer, offset, count, bits);
            return End(crc);
        }

        /// <summary>
        /// Computes a CRC from the UTF8-encoded bytes of a string of text.
        /// </summary>
        /// <param name="text">The text to compute the CRC from.</param>
        /// <returns>The final CRC.</returns>
        public uint Compute(string text)
        {
            return Compute(text, Encoding.UTF8);
        }

        /// <summary>
        /// Computes a CRC from the encoded bytes of a string of text.
        /// </summary>
        /// <param name="text">The text to compute the CRC from.</param>
        /// <param name="encoding">The encoding to use.</param>
        /// <returns>The final CRC.</returns>
        public uint Compute(string text, Encoding encoding)
        {
            if (text == null) { throw new ArgumentNullException("text"); }
            if (encoding == null) { throw new ArgumentNullException("encoding"); }
            return Compute(encoding.GetBytes(text));
        }
    }
}
