﻿#region License
/* NBit Bit Manipulation Library
   Copyright 2012 Illusory Studios LLC <james@illusorystudios.com>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

using System;

namespace NBit
{
    /// <summary>
    /// Computes CRCs.
    /// </summary>
    partial class Crc
    {
        static Crc()
        {
            Parity = Custom(1, 0x1, CrcFormat.LsbFirst, false, false);
            Crc5Usb = Custom(5, 0x05, CrcFormat.LsbFirst, true, true);
            Crc8OneWire = Custom(8, 0x31, CrcFormat.LsbFirst, false, false);
            Crc14Kangaroo = Custom(14, 0x21e8, CrcFormat.LsbFirst | CrcFormat.KoopmanPolynomial, true, true);
            Crc15Can = Custom(15, 0x4599, CrcFormat.MsbFirst, false, false);
            Crc16 = Custom(16, 0x8005, CrcFormat.LsbFirst, false, false);
            Crc16Ccitt = Custom(16, 0x1021, CrcFormat.MsbFirst, true, false);
            Crc16Modbus = Custom(16, 0x8005, CrcFormat.LsbFirst, true, false);
            Crc16XModem = Custom(16, 0x1021, CrcFormat.MsbFirst, false, false);
            Crc32 = Custom(32, 0x04c11db7, CrcFormat.LsbFirst, true, true);
            Crc32Castagnoli = Custom(32, 0x1edc6f41, CrcFormat.LsbFirst, true, true);
        }

        /// <summary>
        /// Creates a custom CRC calculator.
        /// </summary>
        /// <param name="bits">The number of bits in the CRC.</param>
        /// <param name="polynomial">The CRC polynomial.</param>
        /// <param name="format">The CRC format.</param>
        /// <param name="presetToOnes">If true, the initial value will be all ones. Otherwise, it will be all zeros.</param>
        /// <param name="postinvert">If true, the final computed value will be inverted.</param>
        /// <returns>The CRC calculator.</returns>
        public static Crc Custom(int bits, uint polynomial, CrcFormat format, bool presetToOnes, bool postinvert)
        {
            uint mask = bits < 32 ? (1u << bits) - 1 : 0xffffffff;
            return Custom(bits, polynomial, format, presetToOnes ? mask : 0, postinvert ? mask : 0);
        }

        /// <summary>
        /// Creates a custom CRC calculator.
        /// </summary>
        /// <param name="bits">The number of bits in the CRC.</param>
        /// <param name="polynomial">The CRC polynomial.</param>
        /// <param name="format">The CRC format.</param>
        /// <param name="preset">The initial value of the CRC.</param>
        /// <param name="postinvert">A mask of bits to flip when determining the final value.</param>
        /// <returns>The CRC calculator.</returns>
        public static Crc Custom(int bits, uint polynomial, CrcFormat format, uint preset, uint postinvert)
        {
            if (bits < 1 || bits > 32) { throw new ArgumentOutOfRangeException("bits"); }

            if (bits < 32)
            {
                uint notMask = (uint)(-(1u << bits));
                if (0 != (notMask & polynomial)) { throw new ArgumentOutOfRangeException("polynomial"); }
                if (0 != (notMask & preset)) { throw new ArgumentOutOfRangeException("preset"); }
                if (0 != (notMask & postinvert)) { throw new ArgumentOutOfRangeException("postinvert"); }
            }

            switch (format & (CrcFormat.KoopmanPolynomial | CrcFormat.ReversedPolynomial))
            {
                case 0:
                    break;

                case CrcFormat.KoopmanPolynomial:
                    if (0 == (polynomial & (1u << (bits - 1)))) { throw new ArgumentOutOfRangeException("polynomial"); }
                    polynomial &= ~(1u << (bits - 1)); polynomial <<= 1; polynomial |= 1;
                    format &= ~CrcFormat.KoopmanPolynomial;
                    break;

                case CrcFormat.ReversedPolynomial:
                    polynomial = polynomial.ReverseBits(bits);
                    format &= ~CrcFormat.ReversedPolynomial;
                    break;

                default:
                    throw new ArgumentOutOfRangeException("format");
            }

            Crc crc;
            switch (format)
            {
                case CrcFormat.MsbFirst:
                    crc = bits == 16 && polynomial == 0x1021 ? new Crc16Ccitt(preset, postinvert)
                        : new CrcMsb(bits, polynomial, preset, postinvert);
                    break;

                case CrcFormat.LsbFirst:
                    polynomial = polynomial.ReverseBits(bits);
                    crc = new CrcLsb(polynomial, preset, postinvert);
                    break;

                default:
                    throw new ArgumentOutOfRangeException("format");
            }
            crc.Init(); return crc;
        }

        /// <summary>
        /// Simple parity (odd or even bit count).
        /// </summary>
        public static Crc Parity
        {
            get;
            private set;
        }

        /// <summary>
        /// The CRC for USB token packets.
        /// </summary>
        public static Crc Crc5Usb
        {
            get;
            private set;
        }

        /// <summary>
        /// The CRC for the 1-Wire protocol.
        /// </summary>
        public static Crc Crc8OneWire
        {
            get;
            private set;
        }

        /// <summary>
        /// The Modbus CRC, initialized with all zeros.
        /// </summary>
        public static Crc Crc16
        {
            get;
            private set;
        }

        /// <summary>
        /// The CRC for the Kangaroo motion controller.
        /// </summary>
        public static Crc Crc14Kangaroo
        {
            get;
            private set;
        }

        /// <summary>
        /// The CRC for CAN.
        /// </summary>
        public static Crc Crc15Can
        {
            get;
            private set;
        }

        /// <summary>
        /// The CCITT CRC, initialized with all ones.
        /// </summary>
        public static Crc Crc16Ccitt
        {
            get;
            private set;
        }

        /// <summary>
        /// The CRC for the Modbus PLC protocol.
        /// </summary>
        public static Crc Crc16Modbus
        {
            get;
            private set;
        }

        /// <summary>
        /// The CCITT CRC, initialized with all zeros.
        /// </summary>
        public static Crc Crc16XModem
        {
            get;
            private set;
        }

        /// <summary>
        /// Ordinary CRC-32.
        /// </summary>
        public static Crc Crc32
        {
            get;
            private set;
        }

        /// <summary>
        /// CRC-32C, used in SSE4.
        /// </summary>
        public static Crc Crc32Castagnoli
        {
            get;
            private set;
        }
    }
}
